# coding: utf-8
u"""Management-команда для обслуживания системы авторизации."""
from django.core.management import BaseCommand
from django.core.management import CommandError
from termcolor import colored

from educommon.auth.rbac.manager import rbac

from ...models import Permission


class Command(BaseCommand):

    u"""Обслуживание системы авторизации."""

    def _show_permissions(self, registered_only=None):
        u"""Вывод разрешений.

        :param registered_only: определяет, какие разрешения нужно выводить:
            None - все, True - только зарегистрированные в системе, False - все
            разрешения, имеющиеся в БД.
        """
        db_perms = set(Permission.objects.values_list('name', flat=True))
        sys_perms = set(rbac.permissions_by_name.keys())

        if registered_only is None:
            permissions = db_perms | sys_perms
        elif registered_only is True:
            permissions = sys_perms
        elif registered_only is False:
            permissions = db_perms
        else:
            raise TypeError(registered_only)

        if not permissions:
            return

        max_width = max(len(permission) for permission in permissions)
        template = '  {} {:<%s}\n' % max_width

        self.stdout.write('Permissions:\n')

        for permission in sorted(permissions):
            if permission not in db_perms and permission in sys_perms:
                status, color = '[NOT REGISTERED]', 'red'
            elif permission in db_perms and permission not in sys_perms:
                status, color = '[NOT IN SYSTEM ]', 'yellow'
            else:
                status, color = '[  REGISTERED  ]', 'green'

            self.stdout.write(
                colored(template.format(status, permission), color)
            )

    def _show(self, status='all', objects='all', **options):
        u"""Отображение данных системы авторизации."""
        if status == 'all':
            registered_only = None
        elif status == 'registered':
            registered_only = True
        elif status == 'not_registered':
            registered_only = False
        else:
            raise CommandError('Unknown status: {}'.format(status))

        if objects == 'all':
            self._show_permissions(registered_only)
        elif objects == 'permissions':
            self._show_permissions(registered_only)
        else:
            raise CommandError('Unknown objects type: {}'.format(objects))

    def _clean_permissions(self):
        u"""Удаление из БД незарегистрированных в системе разрешений."""
        permissions = Permission.objects.exclude(
            name__in=rbac.permissions_by_name,
        )

        if not permissions:
            return

        max_width = max(len(p.name) for p in permissions)
        template = '{:<%s}  [DELETED]\n' % max_width

        for permission in permissions:
            permission.delete()
            self.stdout.write(
                colored(template.format(permission.name), 'red')
            )

    def _clean(self, objects='all', **options):
        u"""Удаление незарегистрированных правил и разрешений."""
        if objects == 'all':
            self._clean_permissions()
        elif objects == 'permissions':
            self._clean_permissions()
        else:
            raise CommandError('Unknown objects type: {}'.format(objects))

    def handle(self, action, *args, **options):
        # TODO: Написать документацию
        rbac.init(update_db=False)

        if action == 'show':
            self._show(*args, **options)
        elif action == 'clean':
            self._clean(*args, **options)
        # TODO: elif action == 'delete':
        else:
            raise CommandError('Unsupported action: {}'.format(action))
